#!/bin/bash

# test all inputs

NUMTEST=0
FAILS=0
SCORE=0
SOURCEFILE="cbc-ctr.c"
DO_COMPILE=1
DO_CHECKSUMS=1
QUIT_ON_ERROR=0
PAD_ONLY=0
BUILD_ONLY=0
OPTIND=""
IVS="ivs"
SOURCE="input"
KEYS="keys"
ENC="ENCODED"
DEC="DECODED"
GOLD="correct"
PADGOLD="padded"
CHECK_ENCODING=1

function quit_on_fail()
{
	if [[ $1 == 1 ]]
	then
		exit 1
	fi

}

make_checksums()
{
	DIR=$1
	echo "$DIR... "
	pushd $DIR > /dev/null

	VERIFY="checksums/$DIR.verify"

	if [[ -f ../$VERIFY ]]
	then
		rm ../$VERIFY
	fi

	for i in *
	do
		md5sum $i >> "../$VERIFY"
	done

	popd > /dev/null

	if ! diff checksums/$DIR $VERIFY
	then
		echo "--------------------------------------------------"
		echo "                FATAL ERROR!"
		echo "--------------------------------------------------"
		echo "Files in $DIR do not match original test data!"
		echo "You should back up your encoder and decoder source"
		echo "code and re-extract the sample vectors tarball."
		echo "--------------------------------------------------"
		exit 1
	fi
}

usage()
{

	cat <<TEXT
	test.sh -- test your encoder/decoder
	Options:
	-x  Skip compilation step.
	-q  Quit on any error.
	-h  Print this screen.
	-c  Skip checksums check.
	-p  Pad only (no encryption).
	-b  Build only (no testing).
	-E  Skip encoding correctness checks. 

TEXT

	exit 0

}

maybe_quit()
{
	if [[ $QUIT_ON_ERROR == 1 ]]
	then
		if [[ $PAD_ONLY == 1 ]]
		then
			echo "Note: 'pad only' mode was selected."
		fi  
		exit 1
	fi  

}

while getopts "xqhcpbE" OPTION
do
	case $OPTION in
		x)
			DO_COMPILE=0
			echo "Skipping compilation by request."
			;;
		q)
			QUIT_ON_ERROR=1
			echo "Will quit on any error."
			;;
		c)
			DO_CHECKSUMS=0
			echo "Skipping integrity check."
			;;
		p)
			PAD_ONLY=1
			echo "Attempting to perform 'padding only' (requires application support)"
			;;
		b)
			BUILD_ONLY=1
			echo "Will build code only (no testing)."
			;;
		E)
			CHECK_ENCODING=0
			echo "Will compare DECODED to input (but not ENCODING to correct)."
			;;
		h)
			usage
			;;
	esac
done

BANNED="strcpy strcat strtok sprintf vsprintf gets strlen"

# check source code for banned functions
if [[ ! -f $SOURCEFILE ]]
then
	echo "'$SOURCEFILE' is missing!"
	exit 1
else
	echo "Checking for banned functions:"
	for bfunc in $BANNED
	do
		if grep -nE "\s${bfunc}\W" $SOURCEFILE &> /dev/null
		then
			echo "BANNED FUNCTION '$bfunc' DETECTED on line:"
			echo
			grep -nE "\s${bfunc}\W" $SOURCEFILE
			echo
			echo "Remove all banned functions from your code and try again."
			exit 1
		else
			echo "$bfunc"
		fi
	done

	if grep -nE "\sfgets\W" $SOURCEFILE &> /dev/null
	then
		echo 
		echo "WARNING: You almost certainly DO NOT want to use fgets() for this project!"
		echo "Try using something like fgetc() instead!"
		echo
		sleep 10
	fi
fi

# automatically recompile encoder.c and decoder.c UNLESS
# "no_compile" is explicitly requested (FOR OFFICE USE ONLY)
if [[ $DO_COMPILE == 1 ]]
then
	if [[ ! -f $SOURCEFILE ]]
	then
		echo "'$SOURCEFILE' is missing!"
		exit 1
	fi

	if ! gcc -std=c99 $SOURCEFILE -o encoder -D MODE=ENCODE
	then
		echo ":-< : Failed to compile encoder! Quitting."
		exit 1
	else 
		echo ":-D : encoder.c compiled!"
	fi
	
	if ! gcc -std=c99 $SOURCEFILE -o decoder -D MODE=DECODE
	then
		echo ":-< : Failed to compile decoder! Quitting."
		exit 1
	else
		echo ":-D : decoder.c compiled!"
	fi

	if [[ $BUILD_ONLY == 1 ]]
	then
		exit 0
	fi
fi

echo "Removing stale test data (if it exists)..."
rm $ENC/*
rm $DEC/*
rm log/*


if [[ $DO_CHECKSUMS == 1 ]]
then

	echo "Checking integrity of test data:"
	UNAME=$(uname)
	if [[ $UNAME == "Darwin" ]]
	then
		echo "It appears you are using a Mac/BSD."
		echo "This version of the md5 utility has an incompatible output."
		echo "Run these tests on a Linux computer, or disable integrity"
		echo "checking by running this script with the -c flag. See"
		echo "'./test.sh -h' or the README for more information."
		exit 1
	fi

	make_checksums input
	make_checksums correct
	make_checksums keys
	make_checksums ivs
	echo "Done."
fi

for FILEPATH in $SOURCE/*
do
	FILENAME=${FILEPATH##$SOURCE/} # get basename of file

	for KEY in $KEYS/*
	do
		KEYNAME=${KEY##keys/}

		for IV in $IVS/*
		do

			IVNAME=${IV##ivs/}

			for MODE in ECB CBC
			do

				#for ROUNDS in 1 2 4 8 16
				for ROUNDS in 1 2 3 4 5 6 7 8 9 10 11 12 13 14 15 16
				do

					for LOGFILE in log/encoder.log log/encoder.err log/decoder.log log/decoder.err
					do
						echo "----- testing $FILEPATH with key $KEY and iv $IV ($ROUNDS rounds) -----" >> $LOGFILE
					done

					# TEST ENCODING
					
					TGT="$ROUNDS.$KEYNAME.$IVNAME.$FILENAME.$MODE"
					EXEMPLAR="$GOLD/$TGT"
					
					NUMTEST=$((NUMTEST + 1))

					if [[ $MODE == "CBC" ]]
					then
						CMD="./encoder $ROUNDS $KEY $IV $FILEPATH $ENC/$TGT"
					else
						CMD="./encoder $ROUNDS $KEY $IV $FILEPATH $ENC/$TGT 1"
					fi
					
					if ! $CMD >> log/encoder.log 2>> log/encoder.err
					then
						echo -e "\n:-< : Encoder returned non-zero (crashed?)!\n  Encrypting: $FILEPATH \n  with key $KEY and iv $IV!\n  Rounds: $ROUNDS Mode: $MODE"
						echo -e "  Command: '$CMD'\n\n"
						maybe_quit
					fi

					if [[ $CHECK_ENCODING == 1 ]]
					then
						# compare the sample encoded version with your version
						if ! diff $EXEMPLAR $ENC/$TGT &> /dev/null
						then
							echo -e "\n:-( : Encoded file \n\t$ENC/$TGT != $GOLD/$TGT\n"

							FAILS=$((FAILS + 1))
							maybe_quit 
						else 
							echo ":-D : Encoded $FILEPATH with key $KEY and iv $IV matches sample ($ROUNDS rounds). "
						fi
					fi

					# TEST DECODING
					
					NUMTEST=$((NUMTEST + 1))
					EXEMPLAR="$SOURCE/$FILENAME"


					if [[ $MODE == "CBC" ]]
					then
						CMD="./decoder $ROUNDS $KEY $IV $ENC/$TGT $DEC/$TGT"
					else
						CMD="./decoder $ROUNDS $KEY $IV $ENC/$TGT $DEC/$TGT 1"
					fi



					if ! $CMD >> log/decoder.log 2>> log/decoder.err
					then
						echo -e "\n:-< : Decoder returned non-zero (crashed?)!\n  Encrypting: $FILEPATH\n  with key $KEY and iv $IV!\n  Rounds: $ROUNDS mode: $MODE"
						echo -e "  Command: '$CMD'\n\n"
						maybe_quit 
					fi

					# compare the sample decoded version with your decoding

					if ! diff $SOURCE/$FILENAME $DEC/$TGT &> /dev/null
					then

						echo -e "\n:-( : Decoded file \n\t$DEC/$TGT != $SOURCE/$FILENAME!\n\n"
						maybe_quit 

						FAILS=$((FAILS + 1))
					else
						echo ":-D : Decoded $FILEPATH with key $KEY and iv $IV  matches original ($ROUNDS rounds)."
					fi

				done
			done
		done
	done
done

SCORE=$((NUMTEST - FAILS))
echo "You passed $SCORE out of $NUMTEST sample tests."

